@@ -5,12 +5,12 @@
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
+# Foundation; either version 3 of the License, or (at your option) any later
 # version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
+# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
@@ -20,7 +20,10 @@
 
 __metaclass__ = type
 
-from cStringIO import StringIO
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 
 import yaml
 import genshi
@@ -78,11 +81,11 @@ class CairoSerializer:
                                        chart_yaml['options']['height'])
         else:
             raise NotImplementedError
-        
+
         chart = PYCHA_TYPE[chart_info['type']](surface, chart_yaml['options'])
         chart.addDataset(chart_info['dataset'])
         chart.render()
-        
+
         if chart_type == 'png':
             surface.write_to_png(result)
         elif chart_type == 'svg':
