@@ -31,8 +31,12 @@ except ImportError:
 import time
 import urllib
 import zipfile
-from cStringIO import StringIO
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 from copy import deepcopy
+import datetime
 
 
 import warnings
@@ -47,7 +51,8 @@ from genshi.filters.transform import ENTER, EXIT
 from genshi.core import Stream
 from genshi.template.interpolation import PREFIX
 
-
+#from relatorio import __version__ as VERSION
+import relatorio
 from relatorio.templates.base import RelatorioStream
 from relatorio.reporting import Report, MIMETemplateLoader
 try:
@@ -75,6 +80,8 @@ EXTENSIONS = {'image/png': 'png',
 RELATORIO_URI = 'http://relatorio.openhex.org/'
 GENSHI_URI = 'http://genshi.edgewall.org/'
 MANIFEST = 'META-INF/manifest.xml'
+META = 'meta.xml'
+THUMBNAILS = 'Thumbnails'
 output_encode = genshi.output.encode
 EtreeElement = lxml.etree.Element
 
@@ -103,6 +110,7 @@ def guess_type(val):
     elif isinstance(val, (int, float, long)):
         return 'float'
 
+
 class OOTemplateError(genshi.template.base.TemplateSyntaxError):
     "Error to raise when there is a SyntaxError in the genshi template"
 
@@ -169,6 +177,19 @@ class ColumnCounter:
                                         self.counters.get(table_name, 0))
 
 
+class ExpressionCache:
+    """A class used to cache result of expression evaluation"""
+    def __init__(self):
+        self.cache = {}
+
+    def store(self, expression_id, expression):
+        self.cache[expression_id] = expression
+        return expression
+
+    def get(self, expression_id):
+        return self.cache[expression_id]
+
+
 def wrap_nodes_between(first, last, new_parent):
     """An helper function to move all nodes between two nodes to a new node
     and add that new node to their former parent. The boundary nodes are
@@ -189,6 +210,7 @@ def wrap_nodes_between(first, last, new_parent):
         # remove it from its previous parent
         new_parent.append(node)
     old_parent.replace(first, new_parent)
+    new_parent.tail = last.tail
     old_parent.remove(last)
 
 
@@ -366,7 +388,6 @@ class Template(MarkupTemplate):
         office_name = '{%s}value' % self.namespaces['office']
         office_valuetype = '{%s}value-type' % self.namespaces['office']
 
-        py_attrs_attr = '{%s}attrs' % GENSHI_URI
         py_replace = '{%s}replace' % GENSHI_URI
 
         r_statements, closing_tags = self._relatorio_statements(tree)
@@ -429,16 +450,22 @@ class Template(MarkupTemplate):
                                    genshi_node)
             else:
                 # It's not a genshi statement it's a python expression
-                r_node.attrib[py_replace] = expr
                 parent = r_node.getparent().getparent()
                 if parent is None or parent.tag != table_cell_tag:
+                    r_node.attrib[py_replace] = expr
                     continue
 
+                cache_id = id(r_node)
+                r_node.attrib[py_replace] = ("__relatorio_get_cache(%s)" %
+                                             cache_id)
                 # The grand-parent tag is a table cell we should set the
                 # correct value and type for this cell.
-                dico = "{'%s': %s, '%s': __relatorio_guess_type(%s)}"
+                dico = ("{'%s': __relatorio_store_cache(%s, %s), "
+                        "'%s': __relatorio_guess_type("
+                        "__relatorio_get_cache(%s))}")
                 update_py_attrs(parent, dico %
-                        (office_name, expr, office_valuetype, expr))
+                        (office_name, cache_id, expr, office_valuetype,
+                         cache_id))
                 parent.attrib.pop(office_valuetype, None)
                 parent.attrib.pop(office_name, None)
 
@@ -452,7 +479,6 @@ class Template(MarkupTemplate):
         table_col_tag = '{%s}table-column' % table_namespace
         table_num_col_attr = '{%s}number-columns-repeated' % table_namespace
 
-        py_attrs_attr = '{%s}attrs' % GENSHI_URI
         repeat_tag = '{%s}repeat' % RELATORIO_URI
 
         # table node (it is not necessarily the direct parent of ancestor)
@@ -535,11 +561,7 @@ class Template(MarkupTemplate):
                 tag_pos = table_node.index(tag)
                 num = int(tag.attrib.pop(table_num_col_attr))
                 new_tags = [deepcopy(tag) for _ in range(num)]
-                table_node[tag_pos:tag_pos+1] = new_tags
-
-            # recompute the list of column headers as it could
-            # have changed.
-            coldefs = list(table_node.iterchildren(table_col_tag))
+                table_node[tag_pos:tag_pos + 1] = new_tags
 
             # compute the column header nodes corresponding to
             # the opening and closing tags.
@@ -599,7 +621,7 @@ class Template(MarkupTemplate):
         rows_to_wrap = [row_node]
         assert row_node.tag == table_row_tag
         next_rows = row_node.itersiblings(table_row_tag)
-        for row_idx in range(rows_spanned-1):
+        for row_idx in range(rows_spanned - 1):
             next_row_node = next_rows.next()
             rows_to_wrap.append(next_row_node)
             # compute the start and end nodes
@@ -631,17 +653,21 @@ class Template(MarkupTemplate):
         svg_height = '{%s}height' % svg_namespace
         xpath_expr = "//draw:frame[starts-with(@draw:name, 'image:')]"
         for draw in tree.xpath(xpath_expr, namespaces=self.namespaces):
+            cache_id = id(draw)
             d_name = draw.attrib[draw_name][6:].strip()
-            attr_expr = "__relatorio_make_href(%s)" % d_name
+            draw.attrib[draw_name] = ''  # clean template code
+            attr_expr = ("__relatorio_make_href(__relatorio_get_cache(%s))" %
+                         cache_id)
             image_node = EtreeElement(draw_image,
                                       attrib={py_attrs: attr_expr},
                                       nsmap={'draw': draw_namespace,
                                              'py': GENSHI_URI})
             draw.replace(draw[0], image_node)
-            width = draw.attrib.pop(svg_width, None)
-            height = draw.attrib.pop(svg_height, None)
-            attr_expr = "__relatorio_make_dimension(%s, '%s', '%s')" % \
-                    (d_name, width, height)
+            width = draw.attrib.pop(svg_width, '')
+            height = draw.attrib.pop(svg_height, '')
+            attr_expr = ("__relatorio_make_dimension("
+                         "__relatorio_store_cache(%s, %s), '%s', '%s')" %
+                         (cache_id, d_name, width, height))
             draw.attrib[py_attrs] = attr_expr
 
     def _handle_innerdocs(self, tree):
@@ -676,6 +702,10 @@ class Template(MarkupTemplate):
         kwargs['__relatorio_inc_col_count'] = counter.inc
         kwargs['__relatorio_store_col_count'] = counter.store
 
+        cache = ExpressionCache()
+        kwargs['__relatorio_store_cache'] = cache.store
+        kwargs['__relatorio_get_cache'] = cache.get
+
         stream = super(Template, self).generate(*args, **kwargs)
         if self.has_col_loop:
             # Note that we can't simply add a "number-columns-repeated"
@@ -730,20 +760,70 @@ class Manifest(object):
 
     def add_file_entry(self, path, mimetype=None):
         manifest_namespace = self.namespaces['manifest']
-        attribs = {'media-type': mimetype or '',
-                   'full-path': path}
+        attribs = {'{%s}media-type' % manifest_namespace: mimetype or '',
+                   '{%s}full-path' % manifest_namespace: path}
         entry_node = EtreeElement('{%s}%s' % (manifest_namespace,
                                               'file-entry'),
                                   attrib=attribs,
                                   nsmap={'manifest': manifest_namespace})
         self.root.append(entry_node)
 
+    def remove_file_entry(self, path):
+        manifest_namespace = self.namespaces['manifest']
+        entry = self.root.find('{%s}%s[@{%s}full-path="%s"]' %
+                               (manifest_namespace, 'file-entry',
+                                manifest_namespace, path))
+        if entry is not None:
+            self.root.remove(entry)
+
+
+class Meta(object):
+
+    def __init__(self, content):
+        self.tree = lxml.etree.parse(StringIO(content))
+        root = self.tree.getroot()
+        self.namespaces = root.nsmap
+        path = '/office:document-meta/office:meta'
+        self.office_meta, = self.tree.xpath(path, namespaces=self.namespaces)
+
+    def set(self, name, value, namespace='meta'):
+        namespace = self.namespaces[namespace]
+        meta = self.office_meta.find('{%s}%s' % (namespace, name))
+        if meta is None:
+            meta = EtreeElement('{%s}%s' % (namespace, name),
+                                            nsmap={'meta': namespace})
+            self.office_meta.append(meta)
+        meta.text = value
+
+    def remove(self, name, namespace='meta'):
+        namespace = self.namespaces[namespace]
+        meta = self.office_meta.find('{%s}%s' % (namespace, name))
+        if meta is not None:
+            self.office_meta.remove(meta)
+
+    def __str__(self):
+        now = datetime.datetime.now()
+        self.set('creation-date', now.isoformat())
+        self.set('date', now.isoformat())
+        self.remove('document-statistic')
+        self.set('editing-cycles', '1')
+        self.remove('editing-duration')
+        self.set('generator', 'relatorio/%s' % relatorio.__version__)
+        self.remove('initial-creator')
+        self.remove('print-date')
+        self.remove('printed-by')
+        self.remove('creator', 'dc')
+        self.remove('date', 'dc')
+        return lxml.etree.tostring(self.tree, encoding='UTF-8',
+                                   xml_declaration=True)
+
 
 class OOSerializer:
 
     def __init__(self, oo_path):
         self.inzip = zipfile.ZipFile(oo_path)
         self.manifest = Manifest(self.inzip.read(MANIFEST))
+        self.meta = Meta(self.inzip.read(META))
         self.new_oo = StringIO()
         self.outzip = zipfile.ZipFile(self.new_oo, 'w')
         self.xml_serializer = genshi.output.XMLSerializer()
@@ -757,6 +837,7 @@ class OOSerializer:
             files.setdefault(stream_for, []).append((kind, data, pos))
 
         now = time.localtime()[:6]
+        manifest_info = None
         for f_info in self.inzip.infolist():
             if f_info.filename.startswith('ObjectReplacements'):
                 continue
@@ -770,9 +851,16 @@ class OOSerializer:
                 serialized_stream = output_encode(self.xml_serializer(stream))
                 self.outzip.writestr(new_info, serialized_stream)
             elif f_info.filename == MANIFEST:
-                self.outzip.writestr(f_info, str(self.manifest))
+                manifest_info = f_info
+            elif f_info.filename == META:
+                self.outzip.writestr(f_info, str(self.meta))
+            elif f_info.filename.startswith(THUMBNAILS + '/'):
+                self.manifest.remove_file_entry(f_info.filename)
             else:
                 self.outzip.writestr(f_info, self.inzip.read(f_info.filename))
+        self.manifest.remove_file_entry(THUMBNAILS + '/')
+        if manifest_info:
+            self.outzip.writestr(manifest_info, str(self.manifest))
         self.inzip.close()
         self.outzip.close()
 
